function  [F, R] = hhalg(F)
%  [Q, R] = hshldrqr(F)  uses  Householder Reflections  to
%  factorize  F = Q*R  so that  R  is upper-triangular and
%  Q  has orthonormal columns;  Q'*Q = I .  This works only 
%  if  F  has no more columns than rows,  and if underflow
%  does not degrade  F'*F .  Uses  hshldrw.m .
[m, n] = size(F) ;
if (m < n),
    error(' F  has more columns than rows in  hshldrqr(F).'),  end
z = zeros(1, n) ;  w = zeros(m, 1) ;
for  j = 1:n
    [w, z(j)] = hshldrw(F(j:m, j)) ;
    F(j:m, j) = w ;
    if (j < n),
        F(j:m, j+1:n) = F(j:m, j+1:n) - w*(w'*F(j:m, j+1:n)) ;
  end,  end % ... j = 1:n
R = diag(z) + triu(F(1:n, 1:n), 1) ;
for  j = n:-1:1
    w = F(j:m, j) ;  F(:, j) = zeros(m,1) ;  F(j, j) = 1 ;
    F(j:m, j:n) = F(j:m, j:n) - w*(w'*F(j:m, j:n)) ;
  end % ... j = n:-1:1
  
function  [w, z] = hshldrw(x)
%  [w, z] = hshldrw(x)  yields  w  with  w'·w = 2  or  0 ,
%  so  W = I - w*w' = W' = W^-1  reflects the given column 
%  x  to  W*x = [z; 0; 0; ...; 0]  with  |z| = norm(x) .
%  But all bets are off if  UNDERFLOW  degrades  x'*x .
w = x(:) ;  m = length(w) ;  x1 = w(1) ;  a1 = abs(x1) ;
if (m < 2),  w = 0 ;  z = x1 ;  return,  end
if (a1),  s = x1/a1 ;  else  s = 1 ;  end
vv = w(2:m)'*w(2:m) ;  ax = sqrt(a1*a1 + vv) ;
z = -s*ax ;  a1 = a1 + ax ;  w(1) = s*a1 ;
dd2 = a1*ax ;
if (dd2), w = w/sqrt(dd2) ;  end  %...  so  w'*w = 2  unless  w = o .